@@ -79,16 +79,26 @@ class EnergyLabeler:  # pylint: disable=too-many-arguments,  too-many-instance-a
         self.landing_zone_thresholds = landing_zone_thresholds_schema.validate(landing_zone_thresholds)
         self._security_hub_filter = security_hub_filter
         self._frameworks = SecurityHub.validate_frameworks(frameworks)
-        self._landing_zone = LandingZone(landing_zone_name,
-                                         self.landing_zone_thresholds,
-                                         self.account_thresholds,
-                                         allowed_account_ids,
-                                         denied_account_ids)
-        self._security_hub = SecurityHub(region=region,
-                                         allowed_regions=allowed_regions,
-                                         denied_regions=denied_regions)
+        self._landing_zone = self._initialize_landing_zone(landing_zone_name, allowed_account_ids, denied_account_ids)
+        self._security_hub = self._initialize_security_hub(region, allowed_regions, denied_regions)
         self._account_labels_counter = None
         self._query_filter = None
+        self._landing_zone_energy_label = None
+        self._labeled_accounts_energy_label = None
+        self._landing_zone_labeled_accounts = None
+
+    def _initialize_landing_zone(self, name, allowed_account_ids, denied_account_ids):
+        return LandingZone(name,
+                           self.landing_zone_thresholds,
+                           self.account_thresholds,
+                           allowed_account_ids,
+                           denied_account_ids)
+
+    @staticmethod
+    def _initialize_security_hub(region, allowed_regions, denied_regions):
+        return SecurityHub(region=region,
+                           allowed_regions=allowed_regions,
+                           denied_regions=denied_regions)
 
     @property
     def initialized_security_hub_query_filter(self):
@@ -106,6 +116,11 @@ class EnergyLabeler:  # pylint: disable=too-many-arguments,  too-many-instance-a
             self._logger.debug(f'Calculated query {self._query_filter} to execute on security hub.')
         return self._query_filter
 
+    @property
+    def matching_frameworks(self):
+        """The frameworks provided to match the findings of."""
+        return self._frameworks
+
     @property
     def landing_zone(self):
         """Landing zone."""
@@ -120,20 +135,29 @@ class EnergyLabeler:  # pylint: disable=too-many-arguments,  too-many-instance-a
     @cached(cache=TTLCache(maxsize=150000, ttl=120))
     def security_hub_findings(self):
         """Security hub findings."""
-        return self.security_hub.get_findings(self.initialized_security_hub_query_filter)
+        findings = self.security_hub.get_findings(self.initialized_security_hub_query_filter)
+        return self.security_hub.filter_findings_by_frameworks(findings, self._frameworks)
 
     @property
     def landing_zone_energy_label(self):
         """Energy label of the landing zone."""
-        self._logger.debug(f'Landing zone accounts labeled are {len(self._landing_zone.accounts_to_be_labeled)}')
-        return self._landing_zone.get_energy_label(self.security_hub_findings)
+        if self._landing_zone_energy_label is None:
+            self._logger.debug(f'Landing zone accounts labeled are {len(self._landing_zone.accounts_to_be_labeled)}')
+            self._landing_zone_energy_label = self._landing_zone.get_energy_label(self.security_hub_findings)
+        return self._landing_zone_energy_label
 
     @property
     def labeled_accounts_energy_label(self):
         """Energy label of the labeled accounts."""
-        return self._landing_zone.get_energy_label_of_targeted_accounts(self.security_hub_findings)
+        if self._labeled_accounts_energy_label is None:
+            self._labeled_accounts_energy_label = self._landing_zone.get_energy_label_of_targeted_accounts(
+                self.security_hub_findings)
+        return self._labeled_accounts_energy_label
 
     @property
     def landing_zone_labeled_accounts(self):
         """The landing zone labeled account objects."""
-        return self._landing_zone.get_labeled_targeted_accounts(self.security_hub_findings)
+        if self._landing_zone_labeled_accounts is None:
+            self._landing_zone_labeled_accounts = self._landing_zone.get_labeled_targeted_accounts(
+                self.security_hub_findings)
+        return self._landing_zone_labeled_accounts
