@@ -55,7 +55,8 @@ from .awsenergylabelerlibexceptions import (InvalidFrameworks,
                                             NoAccess,
                                             NoRegion,
                                             AccountsNotPartOfLandingZone,
-                                            InvalidPath)
+                                            InvalidPath,
+                                            InvalidRegion)
 from .configuration import (DEFAULT_SECURITY_HUB_FRAMEWORKS,
                             DEFAULT_SECURITY_HUB_FILTER,
                             LANDING_ZONE_THRESHOLDS,
@@ -99,6 +100,7 @@ class LandingZone:  # pylint: disable=too-many-instance-attributes
         self.allowed_account_ids = self._validate_landing_zone_account_ids(allowed_account_ids, account_ids)
         self.denied_account_ids = self._validate_landing_zone_account_ids(denied_account_ids, account_ids)
         self._accounts_to_be_labeled = None
+        self._targeted_accounts_energy_label = None
 
     @staticmethod
     def _validate_landing_zone_account_ids(account_ids, landing_zone_account_ids):
@@ -235,31 +237,34 @@ class LandingZone:  # pylint: disable=too-many-instance-attributes
             energy_label (str): The energy label of the targeted accounts.
 
         """
-        labeled_accounts = self.get_labeled_targeted_accounts(security_hub_findings)
-        label_counter = Counter([account.energy_label.label for account in labeled_accounts])
-        number_of_accounts = len(labeled_accounts)
-        self._logger.debug(f'Number of accounts calculated are {number_of_accounts}')
-        account_sums = []
-        labels = []
-        calculated_label = AggregateAccountsEnergyLabel('F',
-                                                        best_label='F',
-                                                        worst_label='F',
-                                                        accounts_measured=0)
-        for threshold in self.thresholds:
-            label = threshold.get('label')
-            percentage = threshold.get('percentage')
-            labels.append(label)
-            account_sums.append(label_counter.get(label, 0))
-            self._logger.debug(f'Calculating for labels {labels} with threshold {percentage} '
-                               f'and sums of {account_sums}')
-            if sum(account_sums) / number_of_accounts * 100 >= percentage:
-                self._logger.debug(f'Found a match with label {label}')
-                calculated_label = AggregateAccountsEnergyLabel(label,
-                                                                best_label=min(label_counter.keys()),
-                                                                worst_label=max(label_counter.keys()),
-                                                                accounts_measured=number_of_accounts)
-                break
-        return calculated_label
+        if self._targeted_accounts_energy_label is None:
+            labeled_accounts = self.get_labeled_targeted_accounts(security_hub_findings)
+            label_counter = Counter([account.energy_label.label for account in labeled_accounts])
+            number_of_accounts = len(labeled_accounts)
+            self._logger.debug(f'Number of accounts calculated are {number_of_accounts}')
+            account_sums = []
+            labels = []
+            for threshold in self.thresholds:
+                label = threshold.get('label')
+                percentage = threshold.get('percentage')
+                labels.append(label)
+                account_sums.append(label_counter.get(label, 0))
+                self._logger.debug(f'Calculating for labels {labels} with threshold {percentage} '
+                                   f'and sums of {account_sums}')
+                if sum(account_sums) / number_of_accounts * 100 >= percentage:
+                    self._logger.debug(f'Found a match with label {label}')
+                    self._targeted_accounts_energy_label = AggregateAccountsEnergyLabel(label,
+                                                                                        min(label_counter.keys()),
+                                                                                        max(label_counter.keys()),
+                                                                                        number_of_accounts)
+                    break
+            else:
+                self._logger.debug('Found no match with thresholds, using default worst label F.')
+                self._targeted_accounts_energy_label = AggregateAccountsEnergyLabel('F',
+                                                                                    min(label_counter.keys()),
+                                                                                    max(label_counter.keys()),
+                                                                                    number_of_accounts)
+        return self._targeted_accounts_energy_label
 
     def get_energy_label(self, security_hub_findings):
         """Calculates and returns the energy label of the Landing Zone.
@@ -357,6 +362,13 @@ class AwsAccount:
                     self._logger.debug(f'Energy Label for account {self.id} '
                                        f'has been calculated: {self.energy_label.label}')
                     break
+            else:
+                self._logger.debug('No match with thresholds for energy label, using default worst one.')
+                self.energy_label = AccountEnergyLabel('F',
+                                                       number_of_critical_high_findings,
+                                                       number_of_medium_findings,
+                                                       number_of_low_findings,
+                                                       max_days_open)
         except Exception:  # pylint: disable=broad-except
             self._logger.exception(f'Could not calculate energy label for account {self.id}, using the default "F"')
         return self.energy_label
@@ -463,7 +475,7 @@ class Finding:  # pylint: disable=too-many-public-methods
 
     @property
     def is_pci_dss(self):
-        """Is this pic dss framework finding."""
+        """Is this pci dss framework finding."""
         return 'pci-dss/' in self.generator_id
 
     @property
@@ -570,13 +582,26 @@ class SecurityHub:
     def __init__(self, region=None, allowed_regions=None, denied_regions=None):
         self._logger = logging.getLogger(f'{LOGGER_BASENAME}.{self.__class__.__name__}')
         self.allowed_regions, self.denied_regions = validate_allowed_denied_regions(allowed_regions, denied_regions)
-        self.sts = boto3.client('sts')
-        self.ec2 = self._get_client(region)
+        self.sts = self._get_sts_client()
+        self.ec2 = self._get_ec2_client(region)
         self._aws_regions = None
-        self.aws_region = region if region in self.regions else self.sts._client_config.region_name  # noqa
+        self.aws_region = self._validate_region(region) or self._sts_client_config_region
+
+    def _validate_region(self, region):
+        if any([not region, region in self.regions]):
+            return region
+        raise InvalidRegion(region)
+
+    @property
+    def _sts_client_config_region(self):
+        return self.sts._client_config.region_name  # noqa
 
     @staticmethod
-    def _get_client(region):
+    def _get_sts_client():
+        return boto3.client('sts')
+
+    @staticmethod
+    def _get_ec2_client(region):
         kwargs = {}
         if region:
             config = Config(region_name=region)
@@ -592,15 +617,17 @@ class SecurityHub:
             raise InvalidOrNoCredentials(msg) from None
         return client
 
+    def _describe_ec2_regions(self):
+        return self.ec2.describe_regions().get('Regions')
+
     @property
     def regions(self):
         """Regions."""
         if self._aws_regions is None:
             self._aws_regions = [region.get('RegionName')
-                                 for region in self.ec2.describe_regions().get('Regions')
+                                 for region in self._describe_ec2_regions()
                                  if not region.get('OptInStatus', '') == 'not-opted-in']
             self._logger.debug(f'Regions in EC2 that were opted in are : {self._aws_regions}')
-
         if self.allowed_regions:
             self._aws_regions = set(self._aws_regions).intersection(set(self.allowed_regions))
             self._logger.debug(f'Working on allowed regions {self._aws_regions}')
@@ -640,6 +667,26 @@ class SecurityHub:
             self._logger.debug('Could not get aggregating region, either not set, or a client error')
         return aggregating_region
 
+    @staticmethod
+    def filter_findings_by_frameworks(findings, frameworks):
+        """Filters provided findings by the provided frameworks.
+
+        Args:
+            findings: A list containing security hub findings
+            frameworks: The frameworks to filter for
+
+        Returns:
+            findings (list(Findings)): A list of findings matching the provided frameworks
+
+        """
+        frameworks = SecurityHub.validate_frameworks(frameworks)
+
+        def framework_to_finding_attribute(framework):
+            return f'is_{framework.replace("-", "_")}'
+        attributes = [framework_to_finding_attribute(framework) for framework in frameworks]
+        return [finding for finding in findings
+                if any([getattr(finding, attribute) for attribute in attributes])]
+
     @retry(retry_on_exceptions=botocore.exceptions.ClientError)
     def get_findings(self, query_filter):
         """Retrieves findings from security hub.
@@ -659,9 +706,7 @@ class SecurityHub:
             session = boto3.Session(region_name=region)
             security_hub = session.client('securityhub')
             paginator = security_hub.get_paginator('get_findings')
-            iterator = paginator.paginate(
-                Filters=query_filter
-            )
+            iterator = paginator.paginate(Filters=query_filter)
             try:
                 for page in iterator:
                     for finding_data in page['Findings']:
@@ -772,7 +817,7 @@ class DataExporter:  # pylint: disable=too-few-public-methods
         with tempfile.NamedTemporaryFile() as temp_file:
             temp_file.write(data.encode('utf-8'))
             temp_file.flush()
-            dst_filename = urljoin(dst_path, filename)
+            dst_filename = urljoin(dst_path, filename).lstrip("/")
             s3.upload_file(temp_file.name, bucket_name, dst_filename)
             temp_file.close()
         self._logger.info(f'File {filename} copied to {s3_url}')
